(function () {
  "use strict";

  var namespace = require('ns3');
  var path = require('path');
  var child_process = require('child_process');

  var ConsoleScript = function (commands) {
    // Список комманд
    this.commands = commands;
  };

  /**
   * Запуск выполнения комманд
   * @param {string} dir            Рабочая директория (абсолютная или относительная). Default: cwd
   * @param {object} data           Данные замены в коммандах
   * @param {type} cb               Callback(err, stdoutAll, stderr, command)
   * @returns {undefined}
   */
  ConsoleScript.prototype.exec = function (dir, data, cb) {
    var $this = this;
    // Рабочая папка
    var workDirectory = this.getDirectoryPath(dir);
    // Выполняем комманды последовательно
    this._execRecursive(workDirectory, data, function(err, stdoutAll, stderr, index){
      if (err) {
        cb(err, stdoutAll, stderr, $this.commands[index]);
      } else {
        cb(null, stdoutAll, stderr);
      }
    });
  };

  /**
   * Возвращыет абсолютный путь к папке
   * @param {string} dir  (опционально)   Абсолютный или относительный путь
   * @returns {string}      Абсолютный путь. Если dir не задана, то будет возвращена текущая директория
   */
  ConsoleScript.prototype.getDirectoryPath = function (dir) {
    if (dir) {
      if (!path.isAbsolute(dir)) {
        dir = path.join(process.cwd(), dir)
      }
    } else {
      dir = process.cwd();
    }
    return dir;
  };

  /**
   * Запустить все комманды рекурсивно
   * @param {string} workDirectory
   * @param {object} data
   * @param {function} cb           Callback(err, stdoutAll, stderr)
   * @param {integer} index         (оставить undefined)
   * @param {string} stdoutAll      (оставить undefined)
   * @returns {undefined}
   */
  ConsoleScript.prototype._execRecursive = function (workDirectory, data, cb, index, stdoutAll) {
    var $this = this;

    index = index ? index : 0;
    stdoutAll = stdoutAll ? stdoutAll : '';
    var command = this.commands[index];
    
    // Если комманда является исполяемой функцией
    if (typeof command === 'function') {
      return command(function(err, message){
        stdoutAll += message;
        
        if (err) {
          return cb(err, stdoutAll);
        }
        
        index++;
        if (index < $this.commands.length) {
          // Run next command
          $this._execRecursive(workDirectory, data, cb, index, stdoutAll);
        } else {
          // Finished list of commands
          return cb(null, stdoutAll);
        }
      }, workDirectory);
    }
    
    // Комманда
    var cmd = command.cmd;
    cmd = this.stringReplace(cmd, data);
    // Текст ошибки
    var error = command.error ? command.error : false;
    // Timeout
    var timeout = command.timeout ? command.timeout : undefined;
    // Смена директории
    if (command.cd) {
      if (!path.isAbsolute(command.cd)) {
        workDirectory = path.join(workDirectory, command.cd);
      } else {
        workDirectory = command.cd;
      }
    }

    this._execOne(cmd, function (err, stdout, stderr) {

      stdoutAll += stdout;

      if (err) {
        return cb(error ? error : err, stdoutAll, stderr, index);
      }

      index++;
      if (index < $this.commands.length) {
        // Run next command
        $this._execRecursive(workDirectory, data, cb, index, stdoutAll);
      } else {
        // Finished list of commands
        return cb(null, stdoutAll, stderr);
      }

    }, workDirectory, timeout);

  };

  ConsoleScript.prototype.stringReplace = function (str, data) {
    if (data && typeof data === 'object') {
      for (var key in data) {
        str = str.split(key).join(data[key]);
      }
    }
    return str;
  };
  
  /**
   * Вызов одной комманды
   * @param {string} cmd          Комманда
   * @param {function} cb         Callback(err, stdout, stderr)
   * @param {string} cwd          Work directory
   * @param {int} timeout         Timeout, md. Default: 60000
   * @returns {undefined}
   */
  ConsoleScript.prototype._execOne = function (cmd, cb, cwd, timeout) {
    child_process.exec(cmd, {
      encoding: 'utf8',
      killSignal: 'SIGTERM',
      cwd: cwd,
      timeout: timeout ? timeout : 60000,
      env: null
    }, cb);
  };

//  // Usage example:
//  var consoleScript = new ConsoleScript([
//    {cmd: 'git init', timeout: 2000},
//    /**
//     * Middleware async function
//     * @param {function}  next    Middleware callback next(err, message). Message - added to stdout
//     * @param {string} cwd        Work directory
//     * @returns {undefined}
//     */
//    function(next, cwd) {
//      setTimeout(function() {
//        var err = null;
//        var message = "This is text for \"stdoutAll\".\n";
//        // Middleware callback
//        next(null, message);
//      }, 1000);
//    },
//    {cmd: 'git remote add origin {remoteRepoLink}', error: 'Error on git remote add origin.'},
//    {cmd: 'git -c http.sslVerify=false fetch', error: 'Error on git fetch.'},
//    {cmd: 'git checkout -t -f origin/master', error: 'Error on git checkout.'},
//    {cmd: 'git reset --hard HEAD', error: 'Error on git reset head.'},
//    {cmd: 'git archive -o {archivePath} HEAD', error: 'Error on git archive.'},
//    {cd: '..', cmd: 'ls'} // Directory should be changed berore command
//  ]);
//
//  consoleScript.exec('./test', {
//    "{remoteRepoLink}": "https://github.com/pjhl/testp.git",
//    "{archivePath}": "archive.zip"
//  }, function(err, stdoutAll, stderr, command) {
//    if (err) {
//      console.log(err);
//      console.log(command);
//    } else {
//      console.log('~~~OK');
//      console.log(stdoutAll);
//    }
//  });

  namespace('app.libs.ConsoleScript', ConsoleScript);

})();