(function () {
  "use strict";

  var namespace = require('ns3');
  var fs = require('fs');
  var path = require('path');
  var sendFilesHelper = namespace('app.libs.sendFilesHelper');
  var ConsoleScript = namespace('app.libs.ConsoleScript');
  var mktmpdir = require('mktmpdir');
  var AdmZip = require('adm-zip');
  var rimraf = require('rimraf');

  var projectHelper = {
    /**
     * Директория, куда будут складываться bare-репозитории
     */
    rootDir: path.join(__dirname, '../tmp/bares'),
    
    accessTime: {}, // cache_id => unix timestamp
    
    /**
     * Время жизни bare-репозитория проекта, от последнего запроса
     */
    BARE_LIVE_SEC: 120,
    
    get: function (res, project, file, cache_id) {
      var $this = this;
      
      var projectData = namespace('app.config').get('projects.'+project);
      if (projectData) {
        
        // Git repo url, example: "https://github.com/pjhl/testp.git"
        var repoUrl = projectData.url;
        
        var rootHome = '/get/'+project+'/'+cache_id+'/' + (file ? file + '/' : '');
        rootHome = rootHome.replace(/[\/]+$/, '/');
        var dir = path.join(this.rootDir, cache_id);
        var filename = path.join(this.rootDir, cache_id, file);
        
        fs.access(dir, function(err){
          if (err) {
            // Clone repo
            $this.cloneProject(repoUrl, dir, function(err){
              if (err) {
                return res.status(403).send('Error on cloning project.');
              }
              sendFilesHelper.send(res, filename, $this.rootDir, rootHome);
              $this.accessTime[cache_id] = parseInt(new Date().getTime()/1000);
            });
          } else {
            // Проэкт уже должен быть сгенерирован ранее
            sendFilesHelper.send(res, filename, $this.rootDir, rootHome);
            $this.accessTime[cache_id] = parseInt(new Date().getTime()/1000);
          }
        });
        
      } else {
        res.status(404).send('Project not found.');
      }
    },
    
    cloneProject: function(repoUrl, dir, cb) {
      var $this = this;

      mktmpdir(function (err, tmpDir, done) {
        if (err) {
          cb(err);
          return;
        }
        
        var isWin = /^win/.test(process.platform);
        if (isWin) {
          var consoleScript = $this._winConsoleScript();
        } else {
          var consoleScript = $this._linuxConsoleScript();
        }
        consoleScript.exec(tmpDir, {
          "{remoteRepoLink}": repoUrl,
          "{bareDir}": dir
        }, function (err, stdoutAll, stderr, command) {
          // Delete temporary dir
          done();
          
          if (err) {
            console.error(stderr);
            cb(err);
          } else {
            cb();
          }
        });
      });
      
    },
    
    _winConsoleScript: function() {
      return new ConsoleScript([
        {cmd: 'mkdir 1'},
        {cmd: 'mkdir 2'},
        {cd: '1', cmd:'git clone --depth=1 {remoteRepoLink} .'},
        {cmd:'rmdir /s /q .git'},
        {cmd:'git init'},
        {cmd:'git add -A'},
        {cmd:'git commit -m "CIS proxy."'},
        {cmd: 'git archive -o ..\\archive.zip HEAD', error: 'Error on git archive.'},
        {cd: '..', cmd: ''},
        function(next, cwd) {
          var zipFile = path.join(cwd, 'archive.zip');
          var extractTo = path.join(cwd, '2');

          var zip = new AdmZip(zipFile);
          zip.extractAllTo(extractTo);

          next(null, "Archive extracted to \""+extractTo+"\".\n");
        },
        {cmd: 'git clone --bare .\\1 {bareDir} && cd {bareDir} && git update-server-info'}
      ]);
    },
    
    _linuxConsoleScript: function() {
      return new ConsoleScript([
        {cmd: 'mkdir 1'},
        {cmd: 'mkdir 2'},
        {cd: '1', cmd:'git clone --depth=1 {remoteRepoLink} .'},
        {cmd:'rm -rf .git'},
        {cmd:'git init'},
        {cmd:'git add -A'},
        {cmd:'git commit -m \'CIS proxy.\''},
        {cmd: 'git archive -o ../archive.zip HEAD', error: 'Error on git archive.'},
        {cd: '..', cmd: ''},
        function(next, cwd) {
          var zipFile = path.join(cwd, 'archive.zip');
          var extractTo = path.join(cwd, '2');

          var zip = new AdmZip(zipFile);
          zip.extractAllTo(extractTo);

          next(null, "Archive extracted to \""+extractTo+"\".\n");
        },
        {cmd: 'git clone --bare ./1 {bareDir} && cd {bareDir} && git update-server-info'}
      ]);
    },
    
    /**
     * Очистка временны директорий
     * @param {function} cb   Callback(err)
     * @returns {undefined}
     */
    clear: function(cb) {
      rimraf(this.rootDir, cb);
    },
    
    /**
     * Очистка одной временной директории
     * @param {type} cache_id
     * @param {type} cb
     * @returns {undefined}
     */
    remove: function(cache_id, cb) {
      var dir = path.join(this.rootDir, cache_id);
      rimraf(dir, cb);
    },
    
    autoClean: function() {
      var $this = this;
      
      var now = parseInt(new Date().getTime()/1000);
      for (var cache_id in this.accessTime) {
        if (this.accessTime.hasOwnProperty(cache_id)) {
          var time = this.accessTime[cache_id];
          if (now - time > this.BARE_LIVE_SEC) {
            this.remove(cache_id, function(err){
              if (err) {
                console.log('Error on autoClean:', cache_id);
              }
            });
            this.accessTime[cache_id] = false;
          }
        }
      }
      
      for (var cache_id in this.accessTime) {
        if (this.accessTime[cache_id] === false) {
          delete this.accessTime[cache_id];
        }
      }
      
      setTimeout(this.autoClean.bind(this), 4000);
     }
    
  };
  
  // Clear old repositories on start
  projectHelper.clear(function(err){
    if (err) {
      console.log('........INIT CLEAR ............................................. ERROR');
    } else {
      console.log('........INIT CLEAR ............................................. OK');
    }
  });
  
  // Run autoclean
  setTimeout(function(){
    projectHelper.autoClean();
  }, 4000);
  
  namespace('app.libs.projectHelper', projectHelper);

})();